indexing

	description: "Class that implements memory sharing between different%
	%processes on Windows. There is no file underlying this sharing, %
	%it uses the operating-system paging file.%
	%There isn't an immediate POSIX equivalent for this, but you%
	%can use POSIX_MEMORY_MAP and POSIX_SHARED_MEMORY to achieve the%
	%same effect. Implementations of these classes for Windows are coming, %
	%but if you need Windows only, this is more efficient."

	author: "Berend de Boer"
	date: "$Date: 2003/03/20 $"
	revision: "$Revision: #2 $"

class

	WINDOWS_PAGING_FILE_SHARED_MEMORY

inherit

	ABSTRACT_MEMORY_MAP
		redefine
			close,
			dispose
		end

	WINDOWS_BASE


creation

	create_read_write,
	open_read


feature -- Initialization

	create_read_write (a_name: STRING; a_size: INTEGER) is
			-- Create memory mapping without underlying file.
			-- If `name' is Void or empty, the mapping is anonymous.
			-- `name' ordinarily does not have any backslashes in it
			-- except "Global\" or "Local\".
		require
			closed: not is_open
			size_positive: a_size > 0
		do
			handle := posix_createfilemapping (INVALID_HANDLE_VALUE, default_pointer, PAGE_READWRITE, 0, a_size, sh.string_to_pointer (a_name))
			sh.unfreeze_all
			if handle = 0 then
				raise_windows_error
			end
			ptr := posix_mapviewoffile (handle, FILE_MAP_ALL_ACCESS, 0, 0, a_size)
			if ptr = default_pointer then
				raise_windows_error
			end
			capacity := a_size
		ensure
			open: security.error_handling.exceptions_enabled implies is_open
		end

	open_read (a_name: STRING; a_size: INTEGER) is
			-- Open a memory mapping to a named file mapping object.
		require
			name_not_empty: a_name /= Void and then not a_name.is_empty
			size_positive: a_size > 0
		do
			handle := posix_openfilemapping (FILE_MAP_READ, False, sh.string_to_pointer (a_name))
			sh.unfreeze_all
			if handle = 0 then
				raise_windows_error
			end
			ptr := posix_mapviewoffile (handle, FILE_MAP_READ, 0, 0, a_size)
			if ptr = default_pointer then
				raise_windows_error
			end
			capacity := a_size
		ensure
			open: security.error_handling.exceptions_enabled implies is_open
		end

feature -- Cleanup

	dispose is
		local
			b: BOOLEAN
		do
			if ptr /= default_pointer then
				b := posix_unmapviewoffile (ptr)
				b := posix_closehandle (handle)
			end
		rescue
			ptr := default_pointer
			handle := INVALID_HANDLE_VALUE
			retry
		end


feature -- Close

	close is
		do
			safe_win_call (posix_unmapviewoffile (ptr))
			safe_win_call (posix_closehandle (handle))
			precursor
		end

feature {NONE} -- Private state

	handle: INTEGER

end
